# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["ConferenceUpdateParticipantParams"]


class ConferenceUpdateParticipantParams(TypedDict, total=False):
    id: Required[str]

    beep_enabled: Literal["always", "never", "on_enter", "on_exit"]
    """Whether entry/exit beeps are enabled for this participant."""

    end_conference_on_exit: bool
    """Whether the conference should end when this participant exits."""

    soft_end_conference_on_exit: bool
    """Whether the conference should soft-end when this participant exits.

    A soft end will stop new participants from joining but allow existing
    participants to remain.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .runs import (
    RunsResource,
    AsyncRunsResource,
    RunsResourceWithRawResponse,
    AsyncRunsResourceWithRawResponse,
    RunsResourceWithStreamingResponse,
    AsyncRunsResourceWithStreamingResponse,
)
from .test_suites import (
    TestSuitesResource,
    AsyncTestSuitesResource,
    TestSuitesResourceWithRawResponse,
    AsyncTestSuitesResourceWithRawResponse,
    TestSuitesResourceWithStreamingResponse,
    AsyncTestSuitesResourceWithStreamingResponse,
)

__all__ = [
    "RunsResource",
    "AsyncRunsResource",
    "RunsResourceWithRawResponse",
    "AsyncRunsResourceWithRawResponse",
    "RunsResourceWithStreamingResponse",
    "AsyncRunsResourceWithStreamingResponse",
    "TestSuitesResource",
    "AsyncTestSuitesResource",
    "TestSuitesResourceWithRawResponse",
    "AsyncTestSuitesResourceWithRawResponse",
    "TestSuitesResourceWithStreamingResponse",
    "AsyncTestSuitesResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

from .._types import SequenceNotStr

__all__ = ["MediaListParams", "Filter"]


class MediaListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[content_type][]
    """


class Filter(TypedDict, total=False):
    """Consolidated filter parameter (deepObject style).

    Originally: filter[content_type][]
    """

    content_type: SequenceNotStr[str]
    """Filters files by given content types"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.storage import migration_source_create_params
from ...types.storage.migration_source_list_response import MigrationSourceListResponse
from ...types.storage.migration_source_create_response import MigrationSourceCreateResponse
from ...types.storage.migration_source_delete_response import MigrationSourceDeleteResponse
from ...types.storage.migration_source_retrieve_response import MigrationSourceRetrieveResponse

__all__ = ["MigrationSourcesResource", "AsyncMigrationSourcesResource"]


class MigrationSourcesResource(SyncAPIResource):
    """Migrate data from an external provider into Telnyx Cloud Storage"""

    @cached_property
    def with_raw_response(self) -> MigrationSourcesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return MigrationSourcesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> MigrationSourcesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return MigrationSourcesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        bucket_name: str,
        provider: Literal["aws", "telnyx"],
        provider_auth: migration_source_create_params.ProviderAuth,
        source_region: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MigrationSourceCreateResponse:
        """
        Create a source from which data can be migrated from.

        Args:
          bucket_name: Bucket name to migrate the data from.

          provider: Cloud provider from which to migrate data. Use 'telnyx' if you want to migrate
              data from one Telnyx bucket to another.

          source_region: For intra-Telnyx buckets migration, specify the source bucket region in this
              field.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/storage/migration_sources",
            body=maybe_transform(
                {
                    "bucket_name": bucket_name,
                    "provider": provider,
                    "provider_auth": provider_auth,
                    "source_region": source_region,
                },
                migration_source_create_params.MigrationSourceCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MigrationSourceCreateResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MigrationSourceRetrieveResponse:
        """
        Get a Migration Source

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/storage/migration_sources/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MigrationSourceRetrieveResponse,
        )

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MigrationSourceListResponse:
        """List all Migration Sources"""
        return self._get(
            "/storage/migration_sources",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MigrationSourceListResponse,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MigrationSourceDeleteResponse:
        """
        Delete a Migration Source

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/storage/migration_sources/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MigrationSourceDeleteResponse,
        )


class AsyncMigrationSourcesResource(AsyncAPIResource):
    """Migrate data from an external provider into Telnyx Cloud Storage"""

    @cached_property
    def with_raw_response(self) -> AsyncMigrationSourcesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncMigrationSourcesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncMigrationSourcesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncMigrationSourcesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        bucket_name: str,
        provider: Literal["aws", "telnyx"],
        provider_auth: migration_source_create_params.ProviderAuth,
        source_region: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MigrationSourceCreateResponse:
        """
        Create a source from which data can be migrated from.

        Args:
          bucket_name: Bucket name to migrate the data from.

          provider: Cloud provider from which to migrate data. Use 'telnyx' if you want to migrate
              data from one Telnyx bucket to another.

          source_region: For intra-Telnyx buckets migration, specify the source bucket region in this
              field.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/storage/migration_sources",
            body=await async_maybe_transform(
                {
                    "bucket_name": bucket_name,
                    "provider": provider,
                    "provider_auth": provider_auth,
                    "source_region": source_region,
                },
                migration_source_create_params.MigrationSourceCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MigrationSourceCreateResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MigrationSourceRetrieveResponse:
        """
        Get a Migration Source

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/storage/migration_sources/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MigrationSourceRetrieveResponse,
        )

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MigrationSourceListResponse:
        """List all Migration Sources"""
        return await self._get(
            "/storage/migration_sources",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MigrationSourceListResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MigrationSourceDeleteResponse:
        """
        Delete a Migration Source

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/storage/migration_sources/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MigrationSourceDeleteResponse,
        )


class MigrationSourcesResourceWithRawResponse:
    def __init__(self, migration_sources: MigrationSourcesResource) -> None:
        self._migration_sources = migration_sources

        self.create = to_raw_response_wrapper(
            migration_sources.create,
        )
        self.retrieve = to_raw_response_wrapper(
            migration_sources.retrieve,
        )
        self.list = to_raw_response_wrapper(
            migration_sources.list,
        )
        self.delete = to_raw_response_wrapper(
            migration_sources.delete,
        )


class AsyncMigrationSourcesResourceWithRawResponse:
    def __init__(self, migration_sources: AsyncMigrationSourcesResource) -> None:
        self._migration_sources = migration_sources

        self.create = async_to_raw_response_wrapper(
            migration_sources.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            migration_sources.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            migration_sources.list,
        )
        self.delete = async_to_raw_response_wrapper(
            migration_sources.delete,
        )


class MigrationSourcesResourceWithStreamingResponse:
    def __init__(self, migration_sources: MigrationSourcesResource) -> None:
        self._migration_sources = migration_sources

        self.create = to_streamed_response_wrapper(
            migration_sources.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            migration_sources.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            migration_sources.list,
        )
        self.delete = to_streamed_response_wrapper(
            migration_sources.delete,
        )


class AsyncMigrationSourcesResourceWithStreamingResponse:
    def __init__(self, migration_sources: AsyncMigrationSourcesResource) -> None:
        self._migration_sources = migration_sources

        self.create = async_to_streamed_response_wrapper(
            migration_sources.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            migration_sources.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            migration_sources.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            migration_sources.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .sub_number_orders_report import SubNumberOrdersReport

__all__ = ["SubNumberOrdersReportRetrieveResponse"]


class SubNumberOrdersReportRetrieveResponse(BaseModel):
    data: Optional[SubNumberOrdersReport] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .actions import (
    ActionsResource,
    AsyncActionsResource,
    ActionsResourceWithRawResponse,
    AsyncActionsResourceWithRawResponse,
    ActionsResourceWithStreamingResponse,
    AsyncActionsResourceWithStreamingResponse,
)
from .managed_accounts import (
    ManagedAccountsResource,
    AsyncManagedAccountsResource,
    ManagedAccountsResourceWithRawResponse,
    AsyncManagedAccountsResourceWithRawResponse,
    ManagedAccountsResourceWithStreamingResponse,
    AsyncManagedAccountsResourceWithStreamingResponse,
)

__all__ = [
    "ActionsResource",
    "AsyncActionsResource",
    "ActionsResourceWithRawResponse",
    "AsyncActionsResourceWithRawResponse",
    "ActionsResourceWithStreamingResponse",
    "AsyncActionsResourceWithStreamingResponse",
    "ManagedAccountsResource",
    "AsyncManagedAccountsResource",
    "ManagedAccountsResourceWithRawResponse",
    "AsyncManagedAccountsResourceWithRawResponse",
    "ManagedAccountsResourceWithStreamingResponse",
    "AsyncManagedAccountsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, Required, TypedDict

from .call_control_application_inbound_param import CallControlApplicationInboundParam
from .call_control_application_outbound_param import CallControlApplicationOutboundParam

__all__ = ["CallControlApplicationCreateParams"]


class CallControlApplicationCreateParams(TypedDict, total=False):
    application_name: Required[str]
    """A user-assigned name to help manage the application."""

    webhook_event_url: Required[str]
    """The URL where webhooks related to this connection will be sent.

    Must include a scheme, such as 'https'.
    """

    active: bool
    """Specifies whether the connection can be used."""

    anchorsite_override: Literal[
        "Latency",
        "Chicago, IL",
        "Ashburn, VA",
        "San Jose, CA",
        "London, UK",
        "Chennai, IN",
        "Amsterdam, Netherlands",
        "Toronto, Canada",
        "Sydney, Australia",
    ]
    """
    <code>Latency</code> directs Telnyx to route media through the site with the
    lowest round-trip time to the user's connection. Telnyx calculates this time
    using ICMP ping messages. This can be disabled by specifying a site to handle
    all media.
    """

    call_cost_in_webhooks: bool
    """
    Specifies if call cost webhooks should be sent for this Call Control
    Application.
    """

    dtmf_type: Literal["RFC 2833", "Inband", "SIP INFO"]
    """Sets the type of DTMF digits sent from Telnyx to this Connection.

    Note that DTMF digits sent to Telnyx will be accepted in all formats.
    """

    first_command_timeout: bool
    """
    Specifies whether calls to phone numbers associated with this connection should
    hangup after timing out.
    """

    first_command_timeout_secs: int
    """Specifies how many seconds to wait before timing out a dial command."""

    inbound: CallControlApplicationInboundParam

    outbound: CallControlApplicationOutboundParam

    redact_dtmf_debug_logging: bool
    """
    When enabled, DTMF digits entered by users will be redacted in debug logs to
    protect PII data entered through IVR interactions.
    """

    webhook_api_version: Literal["1", "2"]
    """Determines which webhook format will be used, Telnyx API v1 or v2."""

    webhook_event_failover_url: Optional[str]
    """
    The failover URL where webhooks related to this connection will be sent if
    sending to the primary URL fails. Must include a scheme, such as 'https'.
    """

    webhook_timeout_secs: Optional[int]
    """Specifies how many seconds to wait before timing out a webhook."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal

import httpx

from ..types import (
    call_control_application_list_params,
    call_control_application_create_params,
    call_control_application_update_params,
)
from .._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from .._base_client import AsyncPaginator, make_request_options
from ..types.call_control_application import CallControlApplication
from ..types.call_control_application_inbound_param import CallControlApplicationInboundParam
from ..types.call_control_application_outbound_param import CallControlApplicationOutboundParam
from ..types.call_control_application_create_response import CallControlApplicationCreateResponse
from ..types.call_control_application_delete_response import CallControlApplicationDeleteResponse
from ..types.call_control_application_update_response import CallControlApplicationUpdateResponse
from ..types.call_control_application_retrieve_response import CallControlApplicationRetrieveResponse

__all__ = ["CallControlApplicationsResource", "AsyncCallControlApplicationsResource"]


class CallControlApplicationsResource(SyncAPIResource):
    """Call Control applications operations"""

    @cached_property
    def with_raw_response(self) -> CallControlApplicationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return CallControlApplicationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CallControlApplicationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return CallControlApplicationsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        application_name: str,
        webhook_event_url: str,
        active: bool | Omit = omit,
        anchorsite_override: Literal[
            "Latency",
            "Chicago, IL",
            "Ashburn, VA",
            "San Jose, CA",
            "London, UK",
            "Chennai, IN",
            "Amsterdam, Netherlands",
            "Toronto, Canada",
            "Sydney, Australia",
        ]
        | Omit = omit,
        call_cost_in_webhooks: bool | Omit = omit,
        dtmf_type: Literal["RFC 2833", "Inband", "SIP INFO"] | Omit = omit,
        first_command_timeout: bool | Omit = omit,
        first_command_timeout_secs: int | Omit = omit,
        inbound: CallControlApplicationInboundParam | Omit = omit,
        outbound: CallControlApplicationOutboundParam | Omit = omit,
        redact_dtmf_debug_logging: bool | Omit = omit,
        webhook_api_version: Literal["1", "2"] | Omit = omit,
        webhook_event_failover_url: Optional[str] | Omit = omit,
        webhook_timeout_secs: Optional[int] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CallControlApplicationCreateResponse:
        """
        Create a call control application.

        Args:
          application_name: A user-assigned name to help manage the application.

          webhook_event_url: The URL where webhooks related to this connection will be sent. Must include a
              scheme, such as 'https'.

          active: Specifies whether the connection can be used.

          anchorsite_override: <code>Latency</code> directs Telnyx to route media through the site with the
              lowest round-trip time to the user's connection. Telnyx calculates this time
              using ICMP ping messages. This can be disabled by specifying a site to handle
              all media.

          call_cost_in_webhooks: Specifies if call cost webhooks should be sent for this Call Control
              Application.

          dtmf_type: Sets the type of DTMF digits sent from Telnyx to this Connection. Note that DTMF
              digits sent to Telnyx will be accepted in all formats.

          first_command_timeout: Specifies whether calls to phone numbers associated with this connection should
              hangup after timing out.

          first_command_timeout_secs: Specifies how many seconds to wait before timing out a dial command.

          redact_dtmf_debug_logging: When enabled, DTMF digits entered by users will be redacted in debug logs to
              protect PII data entered through IVR interactions.

          webhook_api_version: Determines which webhook format will be used, Telnyx API v1 or v2.

          webhook_event_failover_url: The failover URL where webhooks related to this connection will be sent if
              sending to the primary URL fails. Must include a scheme, such as 'https'.

          webhook_timeout_secs: Specifies how many seconds to wait before timing out a webhook.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/call_control_applications",
            body=maybe_transform(
                {
                    "application_name": application_name,
                    "webhook_event_url": webhook_event_url,
                    "active": active,
                    "anchorsite_override": anchorsite_override,
                    "call_cost_in_webhooks": call_cost_in_webhooks,
                    "dtmf_type": dtmf_type,
                    "first_command_timeout": first_command_timeout,
                    "first_command_timeout_secs": first_command_timeout_secs,
                    "inbound": inbound,
                    "outbound": outbound,
                    "redact_dtmf_debug_logging": redact_dtmf_debug_logging,
                    "webhook_api_version": webhook_api_version,
                    "webhook_event_failover_url": webhook_event_failover_url,
                    "webhook_timeout_secs": webhook_timeout_secs,
                },
                call_control_application_create_params.CallControlApplicationCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CallControlApplicationCreateResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CallControlApplicationRetrieveResponse:
        """
        Retrieves the details of an existing call control application.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/call_control_applications/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CallControlApplicationRetrieveResponse,
        )

    def update(
        self,
        id: str,
        *,
        application_name: str,
        webhook_event_url: str,
        active: bool | Omit = omit,
        anchorsite_override: Literal[
            "Latency",
            "Chicago, IL",
            "Ashburn, VA",
            "San Jose, CA",
            "London, UK",
            "Chennai, IN",
            "Amsterdam, Netherlands",
            "Toronto, Canada",
            "Sydney, Australia",
        ]
        | Omit = omit,
        call_cost_in_webhooks: bool | Omit = omit,
        dtmf_type: Literal["RFC 2833", "Inband", "SIP INFO"] | Omit = omit,
        first_command_timeout: bool | Omit = omit,
        first_command_timeout_secs: int | Omit = omit,
        inbound: CallControlApplicationInboundParam | Omit = omit,
        outbound: CallControlApplicationOutboundParam | Omit = omit,
        redact_dtmf_debug_logging: bool | Omit = omit,
        tags: SequenceNotStr[str] | Omit = omit,
        webhook_api_version: Literal["1", "2"] | Omit = omit,
        webhook_event_failover_url: Optional[str] | Omit = omit,
        webhook_timeout_secs: Optional[int] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CallControlApplicationUpdateResponse:
        """
        Updates settings of an existing call control application.

        Args:
          application_name: A user-assigned name to help manage the application.

          webhook_event_url: The URL where webhooks related to this connection will be sent. Must include a
              scheme, such as 'https'.

          active: Specifies whether the connection can be used.

          anchorsite_override: <code>Latency</code> directs Telnyx to route media through the site with the
              lowest round-trip time to the user's connection. Telnyx calculates this time
              using ICMP ping messages. This can be disabled by specifying a site to handle
              all media.

          call_cost_in_webhooks: Specifies if call cost webhooks should be sent for this Call Control
              Application.

          dtmf_type: Sets the type of DTMF digits sent from Telnyx to this Connection. Note that DTMF
              digits sent to Telnyx will be accepted in all formats.

          first_command_timeout: Specifies whether calls to phone numbers associated with this connection should
              hangup after timing out.

          first_command_timeout_secs: Specifies how many seconds to wait before timing out a dial command.

          redact_dtmf_debug_logging: When enabled, DTMF digits entered by users will be redacted in debug logs to
              protect PII data entered through IVR interactions.

          tags: Tags assigned to the Call Control Application.

          webhook_api_version: Determines which webhook format will be used, Telnyx API v1 or v2.

          webhook_event_failover_url: The failover URL where webhooks related to this connection will be sent if
              sending to the primary URL fails. Must include a scheme, such as 'https'.

          webhook_timeout_secs: Specifies how many seconds to wait before timing out a webhook.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._patch(
            f"/call_control_applications/{id}",
            body=maybe_transform(
                {
                    "application_name": application_name,
                    "webhook_event_url": webhook_event_url,
                    "active": active,
                    "anchorsite_override": anchorsite_override,
                    "call_cost_in_webhooks": call_cost_in_webhooks,
                    "dtmf_type": dtmf_type,
                    "first_command_timeout": first_command_timeout,
                    "first_command_timeout_secs": first_command_timeout_secs,
                    "inbound": inbound,
                    "outbound": outbound,
                    "redact_dtmf_debug_logging": redact_dtmf_debug_logging,
                    "tags": tags,
                    "webhook_api_version": webhook_api_version,
                    "webhook_event_failover_url": webhook_event_failover_url,
                    "webhook_timeout_secs": webhook_timeout_secs,
                },
                call_control_application_update_params.CallControlApplicationUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CallControlApplicationUpdateResponse,
        )

    def list(
        self,
        *,
        filter: call_control_application_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        sort: Literal["created_at", "connection_name", "active"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[CallControlApplication]:
        """
        Return a list of call control applications.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[application_name][contains], filter[outbound.outbound_voice_profile_id],
              filter[leg_id], filter[application_session_id], filter[connection_id],
              filter[product], filter[failed], filter[from], filter[to], filter[name],
              filter[type], filter[occurred_at][eq/gt/gte/lt/lte], filter[status]

          sort: Specifies the sort order for results. By default sorting direction is ascending.
              To have the results sorted in descending order add the <code> -</code>
              prefix.<br/><br/> That is: <ul>

                <li>
                  <code>connection_name</code>: sorts the result by the
                  <code>connection_name</code> field in ascending order.
                </li>

                <li>
                  <code>-connection_name</code>: sorts the result by the
                  <code>connection_name</code> field in descending order.
                </li>
              </ul> <br/> If not given, results are sorted by <code>created_at</code> in descending order.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/call_control_applications",
            page=SyncDefaultFlatPagination[CallControlApplication],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                        "sort": sort,
                    },
                    call_control_application_list_params.CallControlApplicationListParams,
                ),
            ),
            model=CallControlApplication,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CallControlApplicationDeleteResponse:
        """
        Deletes a call control application.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/call_control_applications/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CallControlApplicationDeleteResponse,
        )


class AsyncCallControlApplicationsResource(AsyncAPIResource):
    """Call Control applications operations"""

    @cached_property
    def with_raw_response(self) -> AsyncCallControlApplicationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCallControlApplicationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCallControlApplicationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncCallControlApplicationsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        application_name: str,
        webhook_event_url: str,
        active: bool | Omit = omit,
        anchorsite_override: Literal[
            "Latency",
            "Chicago, IL",
            "Ashburn, VA",
            "San Jose, CA",
            "London, UK",
            "Chennai, IN",
            "Amsterdam, Netherlands",
            "Toronto, Canada",
            "Sydney, Australia",
        ]
        | Omit = omit,
        call_cost_in_webhooks: bool | Omit = omit,
        dtmf_type: Literal["RFC 2833", "Inband", "SIP INFO"] | Omit = omit,
        first_command_timeout: bool | Omit = omit,
        first_command_timeout_secs: int | Omit = omit,
        inbound: CallControlApplicationInboundParam | Omit = omit,
        outbound: CallControlApplicationOutboundParam | Omit = omit,
        redact_dtmf_debug_logging: bool | Omit = omit,
        webhook_api_version: Literal["1", "2"] | Omit = omit,
        webhook_event_failover_url: Optional[str] | Omit = omit,
        webhook_timeout_secs: Optional[int] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CallControlApplicationCreateResponse:
        """
        Create a call control application.

        Args:
          application_name: A user-assigned name to help manage the application.

          webhook_event_url: The URL where webhooks related to this connection will be sent. Must include a
              scheme, such as 'https'.

          active: Specifies whether the connection can be used.

          anchorsite_override: <code>Latency</code> directs Telnyx to route media through the site with the
              lowest round-trip time to the user's connection. Telnyx calculates this time
              using ICMP ping messages. This can be disabled by specifying a site to handle
              all media.

          call_cost_in_webhooks: Specifies if call cost webhooks should be sent for this Call Control
              Application.

          dtmf_type: Sets the type of DTMF digits sent from Telnyx to this Connection. Note that DTMF
              digits sent to Telnyx will be accepted in all formats.

          first_command_timeout: Specifies whether calls to phone numbers associated with this connection should
              hangup after timing out.

          first_command_timeout_secs: Specifies how many seconds to wait before timing out a dial command.

          redact_dtmf_debug_logging: When enabled, DTMF digits entered by users will be redacted in debug logs to
              protect PII data entered through IVR interactions.

          webhook_api_version: Determines which webhook format will be used, Telnyx API v1 or v2.

          webhook_event_failover_url: The failover URL where webhooks related to this connection will be sent if
              sending to the primary URL fails. Must include a scheme, such as 'https'.

          webhook_timeout_secs: Specifies how many seconds to wait before timing out a webhook.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/call_control_applications",
            body=await async_maybe_transform(
                {
                    "application_name": application_name,
                    "webhook_event_url": webhook_event_url,
                    "active": active,
                    "anchorsite_override": anchorsite_override,
                    "call_cost_in_webhooks": call_cost_in_webhooks,
                    "dtmf_type": dtmf_type,
                    "first_command_timeout": first_command_timeout,
                    "first_command_timeout_secs": first_command_timeout_secs,
                    "inbound": inbound,
                    "outbound": outbound,
                    "redact_dtmf_debug_logging": redact_dtmf_debug_logging,
                    "webhook_api_version": webhook_api_version,
                    "webhook_event_failover_url": webhook_event_failover_url,
                    "webhook_timeout_secs": webhook_timeout_secs,
                },
                call_control_application_create_params.CallControlApplicationCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CallControlApplicationCreateResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CallControlApplicationRetrieveResponse:
        """
        Retrieves the details of an existing call control application.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/call_control_applications/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CallControlApplicationRetrieveResponse,
        )

    async def update(
        self,
        id: str,
        *,
        application_name: str,
        webhook_event_url: str,
        active: bool | Omit = omit,
        anchorsite_override: Literal[
            "Latency",
            "Chicago, IL",
            "Ashburn, VA",
            "San Jose, CA",
            "London, UK",
            "Chennai, IN",
            "Amsterdam, Netherlands",
            "Toronto, Canada",
            "Sydney, Australia",
        ]
        | Omit = omit,
        call_cost_in_webhooks: bool | Omit = omit,
        dtmf_type: Literal["RFC 2833", "Inband", "SIP INFO"] | Omit = omit,
        first_command_timeout: bool | Omit = omit,
        first_command_timeout_secs: int | Omit = omit,
        inbound: CallControlApplicationInboundParam | Omit = omit,
        outbound: CallControlApplicationOutboundParam | Omit = omit,
        redact_dtmf_debug_logging: bool | Omit = omit,
        tags: SequenceNotStr[str] | Omit = omit,
        webhook_api_version: Literal["1", "2"] | Omit = omit,
        webhook_event_failover_url: Optional[str] | Omit = omit,
        webhook_timeout_secs: Optional[int] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CallControlApplicationUpdateResponse:
        """
        Updates settings of an existing call control application.

        Args:
          application_name: A user-assigned name to help manage the application.

          webhook_event_url: The URL where webhooks related to this connection will be sent. Must include a
              scheme, such as 'https'.

          active: Specifies whether the connection can be used.

          anchorsite_override: <code>Latency</code> directs Telnyx to route media through the site with the
              lowest round-trip time to the user's connection. Telnyx calculates this time
              using ICMP ping messages. This can be disabled by specifying a site to handle
              all media.

          call_cost_in_webhooks: Specifies if call cost webhooks should be sent for this Call Control
              Application.

          dtmf_type: Sets the type of DTMF digits sent from Telnyx to this Connection. Note that DTMF
              digits sent to Telnyx will be accepted in all formats.

          first_command_timeout: Specifies whether calls to phone numbers associated with this connection should
              hangup after timing out.

          first_command_timeout_secs: Specifies how many seconds to wait before timing out a dial command.

          redact_dtmf_debug_logging: When enabled, DTMF digits entered by users will be redacted in debug logs to
              protect PII data entered through IVR interactions.

          tags: Tags assigned to the Call Control Application.

          webhook_api_version: Determines which webhook format will be used, Telnyx API v1 or v2.

          webhook_event_failover_url: The failover URL where webhooks related to this connection will be sent if
              sending to the primary URL fails. Must include a scheme, such as 'https'.

          webhook_timeout_secs: Specifies how many seconds to wait before timing out a webhook.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._patch(
            f"/call_control_applications/{id}",
            body=await async_maybe_transform(
                {
                    "application_name": application_name,
                    "webhook_event_url": webhook_event_url,
                    "active": active,
                    "anchorsite_override": anchorsite_override,
                    "call_cost_in_webhooks": call_cost_in_webhooks,
                    "dtmf_type": dtmf_type,
                    "first_command_timeout": first_command_timeout,
                    "first_command_timeout_secs": first_command_timeout_secs,
                    "inbound": inbound,
                    "outbound": outbound,
                    "redact_dtmf_debug_logging": redact_dtmf_debug_logging,
                    "tags": tags,
                    "webhook_api_version": webhook_api_version,
                    "webhook_event_failover_url": webhook_event_failover_url,
                    "webhook_timeout_secs": webhook_timeout_secs,
                },
                call_control_application_update_params.CallControlApplicationUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CallControlApplicationUpdateResponse,
        )

    def list(
        self,
        *,
        filter: call_control_application_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        sort: Literal["created_at", "connection_name", "active"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[CallControlApplication, AsyncDefaultFlatPagination[CallControlApplication]]:
        """
        Return a list of call control applications.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[application_name][contains], filter[outbound.outbound_voice_profile_id],
              filter[leg_id], filter[application_session_id], filter[connection_id],
              filter[product], filter[failed], filter[from], filter[to], filter[name],
              filter[type], filter[occurred_at][eq/gt/gte/lt/lte], filter[status]

          sort: Specifies the sort order for results. By default sorting direction is ascending.
              To have the results sorted in descending order add the <code> -</code>
              prefix.<br/><br/> That is: <ul>

                <li>
                  <code>connection_name</code>: sorts the result by the
                  <code>connection_name</code> field in ascending order.
                </li>

                <li>
                  <code>-connection_name</code>: sorts the result by the
                  <code>connection_name</code> field in descending order.
                </li>
              </ul> <br/> If not given, results are sorted by <code>created_at</code> in descending order.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/call_control_applications",
            page=AsyncDefaultFlatPagination[CallControlApplication],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                        "sort": sort,
                    },
                    call_control_application_list_params.CallControlApplicationListParams,
                ),
            ),
            model=CallControlApplication,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CallControlApplicationDeleteResponse:
        """
        Deletes a call control application.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/call_control_applications/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CallControlApplicationDeleteResponse,
        )


class CallControlApplicationsResourceWithRawResponse:
    def __init__(self, call_control_applications: CallControlApplicationsResource) -> None:
        self._call_control_applications = call_control_applications

        self.create = to_raw_response_wrapper(
            call_control_applications.create,
        )
        self.retrieve = to_raw_response_wrapper(
            call_control_applications.retrieve,
        )
        self.update = to_raw_response_wrapper(
            call_control_applications.update,
        )
        self.list = to_raw_response_wrapper(
            call_control_applications.list,
        )
        self.delete = to_raw_response_wrapper(
            call_control_applications.delete,
        )


class AsyncCallControlApplicationsResourceWithRawResponse:
    def __init__(self, call_control_applications: AsyncCallControlApplicationsResource) -> None:
        self._call_control_applications = call_control_applications

        self.create = async_to_raw_response_wrapper(
            call_control_applications.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            call_control_applications.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            call_control_applications.update,
        )
        self.list = async_to_raw_response_wrapper(
            call_control_applications.list,
        )
        self.delete = async_to_raw_response_wrapper(
            call_control_applications.delete,
        )


class CallControlApplicationsResourceWithStreamingResponse:
    def __init__(self, call_control_applications: CallControlApplicationsResource) -> None:
        self._call_control_applications = call_control_applications

        self.create = to_streamed_response_wrapper(
            call_control_applications.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            call_control_applications.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            call_control_applications.update,
        )
        self.list = to_streamed_response_wrapper(
            call_control_applications.list,
        )
        self.delete = to_streamed_response_wrapper(
            call_control_applications.delete,
        )


class AsyncCallControlApplicationsResourceWithStreamingResponse:
    def __init__(self, call_control_applications: AsyncCallControlApplicationsResource) -> None:
        self._call_control_applications = call_control_applications

        self.create = async_to_streamed_response_wrapper(
            call_control_applications.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            call_control_applications.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            call_control_applications.update,
        )
        self.list = async_to_streamed_response_wrapper(
            call_control_applications.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            call_control_applications.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Optional
from datetime import datetime
from typing_extensions import Annotated, TypedDict

from ..._types import SequenceNotStr
from ..._utils import PropertyInfo

__all__ = ["McpServerUpdateParams"]


class McpServerUpdateParams(TypedDict, total=False):
    id: str

    allowed_tools: Optional[SequenceNotStr[str]]

    api_key_ref: Optional[str]

    created_at: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]

    name: str

    type: str

    url: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["BrandTriggerSMSOtpResponse"]


class BrandTriggerSMSOtpResponse(BaseModel):
    """Response after successfully triggering a Brand SMS OTP"""

    brand_id: str = FieldInfo(alias="brandId")
    """The Brand ID for which the OTP was triggered"""

    reference_id: str = FieldInfo(alias="referenceId")
    """The reference ID that can be used to check OTP status"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._types import SequenceNotStr
from ..._utils import PropertyInfo

__all__ = ["MdrUsageReportFetchSyncParams"]


class MdrUsageReportFetchSyncParams(TypedDict, total=False):
    aggregation_type: Required[Literal["NO_AGGREGATION", "PROFILE", "TAGS"]]

    end_date: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]

    profiles: SequenceNotStr[str]

    start_date: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .action_submit_verification_code_params import (
    ActionSubmitVerificationCodeParams as ActionSubmitVerificationCodeParams,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.pagination import SyncDefaultPaginationForQueues, AsyncDefaultPaginationForQueues
from telnyx.types.texml.accounts import (
    QueueListResponse,
    QueueCreateResponse,
    QueueUpdateResponse,
    QueueRetrieveResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestQueues:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create(self, client: Telnyx) -> None:
        queue = client.texml.accounts.queues.create(
            account_sid="account_sid",
        )
        assert_matches_type(QueueCreateResponse, queue, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Telnyx) -> None:
        queue = client.texml.accounts.queues.create(
            account_sid="account_sid",
            friendly_name="Support Queue",
            max_size=10,
        )
        assert_matches_type(QueueCreateResponse, queue, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Telnyx) -> None:
        response = client.texml.accounts.queues.with_raw_response.create(
            account_sid="account_sid",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        queue = response.parse()
        assert_matches_type(QueueCreateResponse, queue, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Telnyx) -> None:
        with client.texml.accounts.queues.with_streaming_response.create(
            account_sid="account_sid",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            queue = response.parse()
            assert_matches_type(QueueCreateResponse, queue, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_create(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_sid` but received ''"):
            client.texml.accounts.queues.with_raw_response.create(
                account_sid="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        queue = client.texml.accounts.queues.retrieve(
            queue_sid="queue_sid",
            account_sid="account_sid",
        )
        assert_matches_type(QueueRetrieveResponse, queue, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.texml.accounts.queues.with_raw_response.retrieve(
            queue_sid="queue_sid",
            account_sid="account_sid",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        queue = response.parse()
        assert_matches_type(QueueRetrieveResponse, queue, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.texml.accounts.queues.with_streaming_response.retrieve(
            queue_sid="queue_sid",
            account_sid="account_sid",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            queue = response.parse()
            assert_matches_type(QueueRetrieveResponse, queue, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_sid` but received ''"):
            client.texml.accounts.queues.with_raw_response.retrieve(
                queue_sid="queue_sid",
                account_sid="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `queue_sid` but received ''"):
            client.texml.accounts.queues.with_raw_response.retrieve(
                queue_sid="",
                account_sid="account_sid",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update(self, client: Telnyx) -> None:
        queue = client.texml.accounts.queues.update(
            queue_sid="queue_sid",
            account_sid="account_sid",
        )
        assert_matches_type(QueueUpdateResponse, queue, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Telnyx) -> None:
        queue = client.texml.accounts.queues.update(
            queue_sid="queue_sid",
            account_sid="account_sid",
            max_size=10,
        )
        assert_matches_type(QueueUpdateResponse, queue, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Telnyx) -> None:
        response = client.texml.accounts.queues.with_raw_response.update(
            queue_sid="queue_sid",
            account_sid="account_sid",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        queue = response.parse()
        assert_matches_type(QueueUpdateResponse, queue, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Telnyx) -> None:
        with client.texml.accounts.queues.with_streaming_response.update(
            queue_sid="queue_sid",
            account_sid="account_sid",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            queue = response.parse()
            assert_matches_type(QueueUpdateResponse, queue, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_sid` but received ''"):
            client.texml.accounts.queues.with_raw_response.update(
                queue_sid="queue_sid",
                account_sid="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `queue_sid` but received ''"):
            client.texml.accounts.queues.with_raw_response.update(
                queue_sid="",
                account_sid="account_sid",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        queue = client.texml.accounts.queues.list(
            account_sid="account_sid",
        )
        assert_matches_type(SyncDefaultPaginationForQueues[QueueListResponse], queue, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        queue = client.texml.accounts.queues.list(
            account_sid="account_sid",
            date_created="DateCreated",
            date_updated="DateUpdated",
            page=0,
            page_size=0,
            page_token="PageToken",
        )
        assert_matches_type(SyncDefaultPaginationForQueues[QueueListResponse], queue, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.texml.accounts.queues.with_raw_response.list(
            account_sid="account_sid",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        queue = response.parse()
        assert_matches_type(SyncDefaultPaginationForQueues[QueueListResponse], queue, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.texml.accounts.queues.with_streaming_response.list(
            account_sid="account_sid",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            queue = response.parse()
            assert_matches_type(SyncDefaultPaginationForQueues[QueueListResponse], queue, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_list(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_sid` but received ''"):
            client.texml.accounts.queues.with_raw_response.list(
                account_sid="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_delete(self, client: Telnyx) -> None:
        queue = client.texml.accounts.queues.delete(
            queue_sid="queue_sid",
            account_sid="account_sid",
        )
        assert queue is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Telnyx) -> None:
        response = client.texml.accounts.queues.with_raw_response.delete(
            queue_sid="queue_sid",
            account_sid="account_sid",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        queue = response.parse()
        assert queue is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Telnyx) -> None:
        with client.texml.accounts.queues.with_streaming_response.delete(
            queue_sid="queue_sid",
            account_sid="account_sid",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            queue = response.parse()
            assert queue is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_sid` but received ''"):
            client.texml.accounts.queues.with_raw_response.delete(
                queue_sid="queue_sid",
                account_sid="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `queue_sid` but received ''"):
            client.texml.accounts.queues.with_raw_response.delete(
                queue_sid="",
                account_sid="account_sid",
            )


class TestAsyncQueues:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncTelnyx) -> None:
        queue = await async_client.texml.accounts.queues.create(
            account_sid="account_sid",
        )
        assert_matches_type(QueueCreateResponse, queue, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncTelnyx) -> None:
        queue = await async_client.texml.accounts.queues.create(
            account_sid="account_sid",
            friendly_name="Support Queue",
            max_size=10,
        )
        assert_matches_type(QueueCreateResponse, queue, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.texml.accounts.queues.with_raw_response.create(
            account_sid="account_sid",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        queue = await response.parse()
        assert_matches_type(QueueCreateResponse, queue, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncTelnyx) -> None:
        async with async_client.texml.accounts.queues.with_streaming_response.create(
            account_sid="account_sid",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            queue = await response.parse()
            assert_matches_type(QueueCreateResponse, queue, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_create(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_sid` but received ''"):
            await async_client.texml.accounts.queues.with_raw_response.create(
                account_sid="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        queue = await async_client.texml.accounts.queues.retrieve(
            queue_sid="queue_sid",
            account_sid="account_sid",
        )
        assert_matches_type(QueueRetrieveResponse, queue, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.texml.accounts.queues.with_raw_response.retrieve(
            queue_sid="queue_sid",
            account_sid="account_sid",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        queue = await response.parse()
        assert_matches_type(QueueRetrieveResponse, queue, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.texml.accounts.queues.with_streaming_response.retrieve(
            queue_sid="queue_sid",
            account_sid="account_sid",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            queue = await response.parse()
            assert_matches_type(QueueRetrieveResponse, queue, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_sid` but received ''"):
            await async_client.texml.accounts.queues.with_raw_response.retrieve(
                queue_sid="queue_sid",
                account_sid="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `queue_sid` but received ''"):
            await async_client.texml.accounts.queues.with_raw_response.retrieve(
                queue_sid="",
                account_sid="account_sid",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncTelnyx) -> None:
        queue = await async_client.texml.accounts.queues.update(
            queue_sid="queue_sid",
            account_sid="account_sid",
        )
        assert_matches_type(QueueUpdateResponse, queue, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncTelnyx) -> None:
        queue = await async_client.texml.accounts.queues.update(
            queue_sid="queue_sid",
            account_sid="account_sid",
            max_size=10,
        )
        assert_matches_type(QueueUpdateResponse, queue, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.texml.accounts.queues.with_raw_response.update(
            queue_sid="queue_sid",
            account_sid="account_sid",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        queue = await response.parse()
        assert_matches_type(QueueUpdateResponse, queue, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncTelnyx) -> None:
        async with async_client.texml.accounts.queues.with_streaming_response.update(
            queue_sid="queue_sid",
            account_sid="account_sid",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            queue = await response.parse()
            assert_matches_type(QueueUpdateResponse, queue, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_sid` but received ''"):
            await async_client.texml.accounts.queues.with_raw_response.update(
                queue_sid="queue_sid",
                account_sid="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `queue_sid` but received ''"):
            await async_client.texml.accounts.queues.with_raw_response.update(
                queue_sid="",
                account_sid="account_sid",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        queue = await async_client.texml.accounts.queues.list(
            account_sid="account_sid",
        )
        assert_matches_type(AsyncDefaultPaginationForQueues[QueueListResponse], queue, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        queue = await async_client.texml.accounts.queues.list(
            account_sid="account_sid",
            date_created="DateCreated",
            date_updated="DateUpdated",
            page=0,
            page_size=0,
            page_token="PageToken",
        )
        assert_matches_type(AsyncDefaultPaginationForQueues[QueueListResponse], queue, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.texml.accounts.queues.with_raw_response.list(
            account_sid="account_sid",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        queue = await response.parse()
        assert_matches_type(AsyncDefaultPaginationForQueues[QueueListResponse], queue, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.texml.accounts.queues.with_streaming_response.list(
            account_sid="account_sid",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            queue = await response.parse()
            assert_matches_type(AsyncDefaultPaginationForQueues[QueueListResponse], queue, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_list(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_sid` but received ''"):
            await async_client.texml.accounts.queues.with_raw_response.list(
                account_sid="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncTelnyx) -> None:
        queue = await async_client.texml.accounts.queues.delete(
            queue_sid="queue_sid",
            account_sid="account_sid",
        )
        assert queue is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.texml.accounts.queues.with_raw_response.delete(
            queue_sid="queue_sid",
            account_sid="account_sid",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        queue = await response.parse()
        assert queue is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncTelnyx) -> None:
        async with async_client.texml.accounts.queues.with_streaming_response.delete(
            queue_sid="queue_sid",
            account_sid="account_sid",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            queue = await response.parse()
            assert queue is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_sid` but received ''"):
            await async_client.texml.accounts.queues.with_raw_response.delete(
                queue_sid="queue_sid",
                account_sid="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `queue_sid` but received ''"):
            await async_client.texml.accounts.queues.with_raw_response.delete(
                queue_sid="",
                account_sid="account_sid",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["QueueUpdateParams"]


class QueueUpdateParams(TypedDict, total=False):
    max_size: Required[int]
    """The maximum number of calls allowed in the queue."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from .actions import (
    ActionsResource,
    AsyncActionsResource,
    ActionsResourceWithRawResponse,
    AsyncActionsResourceWithRawResponse,
    ActionsResourceWithStreamingResponse,
    AsyncActionsResourceWithStreamingResponse,
)
from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import is_given, maybe_transform, strip_not_given, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from ...._base_client import AsyncPaginator, make_request_options
from ....types.organizations import user_list_params, user_retrieve_params
from ....types.organizations.organization_user import OrganizationUser
from ....types.organizations.user_retrieve_response import UserRetrieveResponse
from ....types.organizations.user_get_groups_report_response import UserGetGroupsReportResponse

__all__ = ["UsersResource", "AsyncUsersResource"]


class UsersResource(SyncAPIResource):
    """Operations related to users in your organization"""

    @cached_property
    def actions(self) -> ActionsResource:
        """Operations related to users in your organization"""
        return ActionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> UsersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return UsersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> UsersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return UsersResourceWithStreamingResponse(self)

    def retrieve(
        self,
        id: str,
        *,
        include_groups: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UserRetrieveResponse:
        """
        Returns a user in your organization.

        Args:
          include_groups: When set to true, includes the groups array for each user in the response. The
              groups array contains objects with id and name for each group the user belongs
              to.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/organizations/users/{id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"include_groups": include_groups}, user_retrieve_params.UserRetrieveParams),
            ),
            cast_to=UserRetrieveResponse,
        )

    def list(
        self,
        *,
        filter_email: str | Omit = omit,
        filter_user_status: Literal["enabled", "disabled", "blocked"] | Omit = omit,
        include_groups: bool | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[OrganizationUser]:
        """
        Returns a list of the users in your organization.

        Args:
          filter_email: Filter by email address (partial match)

          filter_user_status: Filter by user status

          include_groups: When set to true, includes the groups array for each user in the response. The
              groups array contains objects with id and name for each group the user belongs
              to.

          page_number: The page number to load

          page_size: The size of the page

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/organizations/users",
            page=SyncDefaultFlatPagination[OrganizationUser],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter_email": filter_email,
                        "filter_user_status": filter_user_status,
                        "include_groups": include_groups,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    user_list_params.UserListParams,
                ),
            ),
            model=OrganizationUser,
        )

    def get_groups_report(
        self,
        *,
        accept: Literal["application/json", "text/csv"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UserGetGroupsReportResponse:
        """
        Returns a report of all users in your organization with their group memberships.
        This endpoint returns all users without pagination and always includes group
        information. The report can be retrieved in JSON or CSV format by sending
        specific content-type headers.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {
            **strip_not_given({"Accept": str(accept) if is_given(accept) else not_given}),
            **(extra_headers or {}),
        }
        return self._get(
            "/organizations/users/users_groups_report",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UserGetGroupsReportResponse,
        )


class AsyncUsersResource(AsyncAPIResource):
    """Operations related to users in your organization"""

    @cached_property
    def actions(self) -> AsyncActionsResource:
        """Operations related to users in your organization"""
        return AsyncActionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncUsersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncUsersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncUsersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncUsersResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        id: str,
        *,
        include_groups: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UserRetrieveResponse:
        """
        Returns a user in your organization.

        Args:
          include_groups: When set to true, includes the groups array for each user in the response. The
              groups array contains objects with id and name for each group the user belongs
              to.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/organizations/users/{id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"include_groups": include_groups}, user_retrieve_params.UserRetrieveParams
                ),
            ),
            cast_to=UserRetrieveResponse,
        )

    def list(
        self,
        *,
        filter_email: str | Omit = omit,
        filter_user_status: Literal["enabled", "disabled", "blocked"] | Omit = omit,
        include_groups: bool | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[OrganizationUser, AsyncDefaultFlatPagination[OrganizationUser]]:
        """
        Returns a list of the users in your organization.

        Args:
          filter_email: Filter by email address (partial match)

          filter_user_status: Filter by user status

          include_groups: When set to true, includes the groups array for each user in the response. The
              groups array contains objects with id and name for each group the user belongs
              to.

          page_number: The page number to load

          page_size: The size of the page

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/organizations/users",
            page=AsyncDefaultFlatPagination[OrganizationUser],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter_email": filter_email,
                        "filter_user_status": filter_user_status,
                        "include_groups": include_groups,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    user_list_params.UserListParams,
                ),
            ),
            model=OrganizationUser,
        )

    async def get_groups_report(
        self,
        *,
        accept: Literal["application/json", "text/csv"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UserGetGroupsReportResponse:
        """
        Returns a report of all users in your organization with their group memberships.
        This endpoint returns all users without pagination and always includes group
        information. The report can be retrieved in JSON or CSV format by sending
        specific content-type headers.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {
            **strip_not_given({"Accept": str(accept) if is_given(accept) else not_given}),
            **(extra_headers or {}),
        }
        return await self._get(
            "/organizations/users/users_groups_report",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UserGetGroupsReportResponse,
        )


class UsersResourceWithRawResponse:
    def __init__(self, users: UsersResource) -> None:
        self._users = users

        self.retrieve = to_raw_response_wrapper(
            users.retrieve,
        )
        self.list = to_raw_response_wrapper(
            users.list,
        )
        self.get_groups_report = to_raw_response_wrapper(
            users.get_groups_report,
        )

    @cached_property
    def actions(self) -> ActionsResourceWithRawResponse:
        """Operations related to users in your organization"""
        return ActionsResourceWithRawResponse(self._users.actions)


class AsyncUsersResourceWithRawResponse:
    def __init__(self, users: AsyncUsersResource) -> None:
        self._users = users

        self.retrieve = async_to_raw_response_wrapper(
            users.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            users.list,
        )
        self.get_groups_report = async_to_raw_response_wrapper(
            users.get_groups_report,
        )

    @cached_property
    def actions(self) -> AsyncActionsResourceWithRawResponse:
        """Operations related to users in your organization"""
        return AsyncActionsResourceWithRawResponse(self._users.actions)


class UsersResourceWithStreamingResponse:
    def __init__(self, users: UsersResource) -> None:
        self._users = users

        self.retrieve = to_streamed_response_wrapper(
            users.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            users.list,
        )
        self.get_groups_report = to_streamed_response_wrapper(
            users.get_groups_report,
        )

    @cached_property
    def actions(self) -> ActionsResourceWithStreamingResponse:
        """Operations related to users in your organization"""
        return ActionsResourceWithStreamingResponse(self._users.actions)


class AsyncUsersResourceWithStreamingResponse:
    def __init__(self, users: AsyncUsersResource) -> None:
        self._users = users

        self.retrieve = async_to_streamed_response_wrapper(
            users.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            users.list,
        )
        self.get_groups_report = async_to_streamed_response_wrapper(
            users.get_groups_report,
        )

    @cached_property
    def actions(self) -> AsyncActionsResourceWithStreamingResponse:
        """Operations related to users in your organization"""
        return AsyncActionsResourceWithStreamingResponse(self._users.actions)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import Literal

import httpx

from ....._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ....._utils import maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .....pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from ....._base_client import AsyncPaginator, make_request_options
from .....types.ai.missions.runs import event_log_params, event_list_params
from .....types.ai.missions.runs.event_data import EventData
from .....types.ai.missions.runs.event_log_response import EventLogResponse
from .....types.ai.missions.runs.event_get_event_details_response import EventGetEventDetailsResponse

__all__ = ["EventsResource", "AsyncEventsResource"]


class EventsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> EventsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return EventsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> EventsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return EventsResourceWithStreamingResponse(self)

    def list(
        self,
        run_id: str,
        *,
        mission_id: str,
        agent_id: str | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        step_id: str | Omit = omit,
        type: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[EventData]:
        """
        List events for a run (paginated)

        Args:
          page_number: Page number (1-based)

          page_size: Number of items per page

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        if not run_id:
            raise ValueError(f"Expected a non-empty value for `run_id` but received {run_id!r}")
        return self._get_api_list(
            f"/ai/missions/{mission_id}/runs/{run_id}/events",
            page=SyncDefaultFlatPagination[EventData],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "agent_id": agent_id,
                        "page_number": page_number,
                        "page_size": page_size,
                        "step_id": step_id,
                        "type": type,
                    },
                    event_list_params.EventListParams,
                ),
            ),
            model=EventData,
        )

    def get_event_details(
        self,
        event_id: str,
        *,
        mission_id: str,
        run_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EventGetEventDetailsResponse:
        """
        Get details of a specific event

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        if not run_id:
            raise ValueError(f"Expected a non-empty value for `run_id` but received {run_id!r}")
        if not event_id:
            raise ValueError(f"Expected a non-empty value for `event_id` but received {event_id!r}")
        return self._get(
            f"/ai/missions/{mission_id}/runs/{run_id}/events/{event_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EventGetEventDetailsResponse,
        )

    def log(
        self,
        run_id: str,
        *,
        mission_id: str,
        summary: str,
        type: Literal[
            "status_change",
            "step_started",
            "step_completed",
            "step_failed",
            "tool_call",
            "tool_result",
            "message",
            "error",
            "custom",
        ],
        agent_id: str | Omit = omit,
        idempotency_key: str | Omit = omit,
        payload: Dict[str, object] | Omit = omit,
        step_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EventLogResponse:
        """
        Log an event for a run

        Args:
          idempotency_key: Prevents duplicate events on retry

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        if not run_id:
            raise ValueError(f"Expected a non-empty value for `run_id` but received {run_id!r}")
        return self._post(
            f"/ai/missions/{mission_id}/runs/{run_id}/events",
            body=maybe_transform(
                {
                    "summary": summary,
                    "type": type,
                    "agent_id": agent_id,
                    "idempotency_key": idempotency_key,
                    "payload": payload,
                    "step_id": step_id,
                },
                event_log_params.EventLogParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EventLogResponse,
        )


class AsyncEventsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncEventsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncEventsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncEventsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncEventsResourceWithStreamingResponse(self)

    def list(
        self,
        run_id: str,
        *,
        mission_id: str,
        agent_id: str | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        step_id: str | Omit = omit,
        type: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[EventData, AsyncDefaultFlatPagination[EventData]]:
        """
        List events for a run (paginated)

        Args:
          page_number: Page number (1-based)

          page_size: Number of items per page

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        if not run_id:
            raise ValueError(f"Expected a non-empty value for `run_id` but received {run_id!r}")
        return self._get_api_list(
            f"/ai/missions/{mission_id}/runs/{run_id}/events",
            page=AsyncDefaultFlatPagination[EventData],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "agent_id": agent_id,
                        "page_number": page_number,
                        "page_size": page_size,
                        "step_id": step_id,
                        "type": type,
                    },
                    event_list_params.EventListParams,
                ),
            ),
            model=EventData,
        )

    async def get_event_details(
        self,
        event_id: str,
        *,
        mission_id: str,
        run_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EventGetEventDetailsResponse:
        """
        Get details of a specific event

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        if not run_id:
            raise ValueError(f"Expected a non-empty value for `run_id` but received {run_id!r}")
        if not event_id:
            raise ValueError(f"Expected a non-empty value for `event_id` but received {event_id!r}")
        return await self._get(
            f"/ai/missions/{mission_id}/runs/{run_id}/events/{event_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EventGetEventDetailsResponse,
        )

    async def log(
        self,
        run_id: str,
        *,
        mission_id: str,
        summary: str,
        type: Literal[
            "status_change",
            "step_started",
            "step_completed",
            "step_failed",
            "tool_call",
            "tool_result",
            "message",
            "error",
            "custom",
        ],
        agent_id: str | Omit = omit,
        idempotency_key: str | Omit = omit,
        payload: Dict[str, object] | Omit = omit,
        step_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EventLogResponse:
        """
        Log an event for a run

        Args:
          idempotency_key: Prevents duplicate events on retry

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        if not run_id:
            raise ValueError(f"Expected a non-empty value for `run_id` but received {run_id!r}")
        return await self._post(
            f"/ai/missions/{mission_id}/runs/{run_id}/events",
            body=await async_maybe_transform(
                {
                    "summary": summary,
                    "type": type,
                    "agent_id": agent_id,
                    "idempotency_key": idempotency_key,
                    "payload": payload,
                    "step_id": step_id,
                },
                event_log_params.EventLogParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EventLogResponse,
        )


class EventsResourceWithRawResponse:
    def __init__(self, events: EventsResource) -> None:
        self._events = events

        self.list = to_raw_response_wrapper(
            events.list,
        )
        self.get_event_details = to_raw_response_wrapper(
            events.get_event_details,
        )
        self.log = to_raw_response_wrapper(
            events.log,
        )


class AsyncEventsResourceWithRawResponse:
    def __init__(self, events: AsyncEventsResource) -> None:
        self._events = events

        self.list = async_to_raw_response_wrapper(
            events.list,
        )
        self.get_event_details = async_to_raw_response_wrapper(
            events.get_event_details,
        )
        self.log = async_to_raw_response_wrapper(
            events.log,
        )


class EventsResourceWithStreamingResponse:
    def __init__(self, events: EventsResource) -> None:
        self._events = events

        self.list = to_streamed_response_wrapper(
            events.list,
        )
        self.get_event_details = to_streamed_response_wrapper(
            events.get_event_details,
        )
        self.log = to_streamed_response_wrapper(
            events.log,
        )


class AsyncEventsResourceWithStreamingResponse:
    def __init__(self, events: AsyncEventsResource) -> None:
        self._events = events

        self.list = async_to_streamed_response_wrapper(
            events.list,
        )
        self.get_event_details = async_to_streamed_response_wrapper(
            events.get_event_details,
        )
        self.log = async_to_streamed_response_wrapper(
            events.log,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ...._models import BaseModel

__all__ = ["EmbeddingCreateEmbeddingsResponse", "Data", "Usage"]


class Data(BaseModel):
    embedding: List[float]
    """The embedding vector"""

    index: int
    """The index of the embedding in the list of embeddings"""

    object: str
    """The object type, always 'embedding'"""


class Usage(BaseModel):
    prompt_tokens: int
    """Number of tokens in the input"""

    total_tokens: int
    """Total number of tokens used"""


class EmbeddingCreateEmbeddingsResponse(BaseModel):
    data: List[Data]
    """List of embedding objects"""

    model: str
    """The model used for embedding"""

    object: str
    """The object type, always 'list'"""

    usage: Usage



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .queue import Queue
from .._models import BaseModel

__all__ = ["QueueCreateResponse"]


class QueueCreateResponse(BaseModel):
    data: Optional[Queue] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["SimCardGroupUpdateParams", "DataLimit"]


class SimCardGroupUpdateParams(TypedDict, total=False):
    data_limit: DataLimit
    """Upper limit on the amount of data the SIM cards, within the group, can use."""

    name: str
    """A user friendly name for the SIM card group."""


class DataLimit(TypedDict, total=False):
    """Upper limit on the amount of data the SIM cards, within the group, can use."""

    amount: str

    unit: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["CommentCreateParams"]


class CommentCreateParams(TypedDict, total=False):
    body: str

    comment_record_id: str

    comment_record_type: Literal["sub_number_order", "requirement_group"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["SimCardGroupRetrieveParams"]


class SimCardGroupRetrieveParams(TypedDict, total=False):
    include_iccids: bool
    """It includes a list of associated ICCIDs."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel
from .civic_address import CivicAddress

__all__ = ["CivicAddressListResponse"]


class CivicAddressListResponse(BaseModel):
    data: Optional[List[CivicAddress]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["ManagedAccountUpdateGlobalChannelLimitParams"]


class ManagedAccountUpdateGlobalChannelLimitParams(TypedDict, total=False):
    channel_limit: int
    """
    Integer value that indicates the number of allocatable global outbound channels
    that should be allocated to the managed account. Must be 0 or more. If the value
    is 0 then the account will have no usable channels and will not be able to
    perform outbound calling.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["ConnectionRtcpSettings"]


class ConnectionRtcpSettings(BaseModel):
    capture_enabled: Optional[bool] = None
    """
    BETA - Enable the capture and storage of RTCP messages to create QoS reports on
    the Telnyx Mission Control Portal.
    """

    port: Optional[Literal["rtcp-mux", "rtp+1"]] = None
    """RTCP port by default is rtp+1, it can also be set to rtcp-mux"""

    report_frequency_secs: Optional[int] = None
    """RTCP reports are sent to customers based on the frequency set.

    Frequency is in seconds and it can be set to values from 5 to 3000 seconds.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["ActionLeaveParams"]


class ActionLeaveParams(TypedDict, total=False):
    call_control_id: Required[str]
    """Unique identifier and token for controlling the call"""

    beep_enabled: Literal["always", "never", "on_enter", "on_exit"]
    """Whether a beep sound should be played when the participant leaves the
    conference.

    Can be used to override the conference-level setting.
    """

    command_id: str
    """Use this field to avoid execution of duplicate commands.

    Telnyx will ignore subsequent commands with the same `command_id` as one that
    has already been executed.
    """

    region: Literal["Australia", "Europe", "Middle East", "US"]
    """Region where the conference data is located.

    Defaults to the region defined in user's data locality settings (Europe or US).
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from ..._models import BaseModel
from .background_task_status import BackgroundTaskStatus

__all__ = ["EmbeddingListResponse", "Data"]


class Data(BaseModel):
    created_at: datetime

    status: BackgroundTaskStatus
    """Status of an embeddings task."""

    task_id: str

    task_name: str

    user_id: str

    bucket: Optional[str] = None

    finished_at: Optional[datetime] = None


class EmbeddingListResponse(BaseModel):
    data: List[Data]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ...._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from ...._base_client import AsyncPaginator, make_request_options
from ....types.whatsapp.business_accounts import phone_number_list_params, phone_number_initialize_verification_params
from ....types.whatsapp.business_accounts.phone_number_list_response import PhoneNumberListResponse

__all__ = ["PhoneNumbersResource", "AsyncPhoneNumbersResource"]


class PhoneNumbersResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> PhoneNumbersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return PhoneNumbersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PhoneNumbersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return PhoneNumbersResourceWithStreamingResponse(self)

    def list(
        self,
        id: str,
        *,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[PhoneNumberListResponse]:
        """
        List phone numbers for a WABA

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get_api_list(
            f"/v2/whatsapp/business_accounts/{id}/phone_numbers",
            page=SyncDefaultFlatPagination[PhoneNumberListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    phone_number_list_params.PhoneNumberListParams,
                ),
            ),
            model=PhoneNumberListResponse,
        )

    def initialize_verification(
        self,
        id: str,
        *,
        display_name: str,
        phone_number: str,
        language: str | Omit = omit,
        verification_method: Literal["sms", "voice"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Initialize Whatsapp phone number verification

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._post(
            f"/v2/whatsapp/business_accounts/{id}/phone_numbers",
            body=maybe_transform(
                {
                    "display_name": display_name,
                    "phone_number": phone_number,
                    "language": language,
                    "verification_method": verification_method,
                },
                phone_number_initialize_verification_params.PhoneNumberInitializeVerificationParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncPhoneNumbersResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncPhoneNumbersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPhoneNumbersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPhoneNumbersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncPhoneNumbersResourceWithStreamingResponse(self)

    def list(
        self,
        id: str,
        *,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[PhoneNumberListResponse, AsyncDefaultFlatPagination[PhoneNumberListResponse]]:
        """
        List phone numbers for a WABA

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get_api_list(
            f"/v2/whatsapp/business_accounts/{id}/phone_numbers",
            page=AsyncDefaultFlatPagination[PhoneNumberListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    phone_number_list_params.PhoneNumberListParams,
                ),
            ),
            model=PhoneNumberListResponse,
        )

    async def initialize_verification(
        self,
        id: str,
        *,
        display_name: str,
        phone_number: str,
        language: str | Omit = omit,
        verification_method: Literal["sms", "voice"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Initialize Whatsapp phone number verification

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._post(
            f"/v2/whatsapp/business_accounts/{id}/phone_numbers",
            body=await async_maybe_transform(
                {
                    "display_name": display_name,
                    "phone_number": phone_number,
                    "language": language,
                    "verification_method": verification_method,
                },
                phone_number_initialize_verification_params.PhoneNumberInitializeVerificationParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class PhoneNumbersResourceWithRawResponse:
    def __init__(self, phone_numbers: PhoneNumbersResource) -> None:
        self._phone_numbers = phone_numbers

        self.list = to_raw_response_wrapper(
            phone_numbers.list,
        )
        self.initialize_verification = to_raw_response_wrapper(
            phone_numbers.initialize_verification,
        )


class AsyncPhoneNumbersResourceWithRawResponse:
    def __init__(self, phone_numbers: AsyncPhoneNumbersResource) -> None:
        self._phone_numbers = phone_numbers

        self.list = async_to_raw_response_wrapper(
            phone_numbers.list,
        )
        self.initialize_verification = async_to_raw_response_wrapper(
            phone_numbers.initialize_verification,
        )


class PhoneNumbersResourceWithStreamingResponse:
    def __init__(self, phone_numbers: PhoneNumbersResource) -> None:
        self._phone_numbers = phone_numbers

        self.list = to_streamed_response_wrapper(
            phone_numbers.list,
        )
        self.initialize_verification = to_streamed_response_wrapper(
            phone_numbers.initialize_verification,
        )


class AsyncPhoneNumbersResourceWithStreamingResponse:
    def __init__(self, phone_numbers: AsyncPhoneNumbersResource) -> None:
        self._phone_numbers = phone_numbers

        self.list = async_to_streamed_response_wrapper(
            phone_numbers.list,
        )
        self.initialize_verification = async_to_streamed_response_wrapper(
            phone_numbers.initialize_verification,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Mapping, cast

import httpx

from ....._types import Body, Query, Headers, NoneType, NotGiven, FileTypes, not_given
from ....._utils import extract_files, maybe_transform, deepcopy_minimal, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._base_client import make_request_options
from .....types.whatsapp.phone_numbers.profile import photo_upload_params
from .....types.whatsapp.phone_numbers.profile.photo_upload_response import PhotoUploadResponse

__all__ = ["PhotoResource", "AsyncPhotoResource"]


class PhotoResource(SyncAPIResource):
    """Manage Whatsapp phone numbers"""

    @cached_property
    def with_raw_response(self) -> PhotoResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return PhotoResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PhotoResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return PhotoResourceWithStreamingResponse(self)

    def delete(
        self,
        phone_number: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete Whatsapp profile photo

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not phone_number:
            raise ValueError(f"Expected a non-empty value for `phone_number` but received {phone_number!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/v2/whatsapp/phone_numbers/{phone_number}/profile/photo",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def upload(
        self,
        phone_number: str,
        *,
        file: FileTypes,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PhotoUploadResponse:
        """
        Upload Whatsapp profile photo

        Args:
          file: Image file (JPEG recommended, max 10 MB)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not phone_number:
            raise ValueError(f"Expected a non-empty value for `phone_number` but received {phone_number!r}")
        body = deepcopy_minimal({"file": file})
        files = extract_files(cast(Mapping[str, object], body), paths=[["file"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return self._post(
            f"/v2/whatsapp/phone_numbers/{phone_number}/profile/photo",
            body=maybe_transform(body, photo_upload_params.PhotoUploadParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PhotoUploadResponse,
        )


class AsyncPhotoResource(AsyncAPIResource):
    """Manage Whatsapp phone numbers"""

    @cached_property
    def with_raw_response(self) -> AsyncPhotoResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPhotoResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPhotoResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncPhotoResourceWithStreamingResponse(self)

    async def delete(
        self,
        phone_number: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete Whatsapp profile photo

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not phone_number:
            raise ValueError(f"Expected a non-empty value for `phone_number` but received {phone_number!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/v2/whatsapp/phone_numbers/{phone_number}/profile/photo",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def upload(
        self,
        phone_number: str,
        *,
        file: FileTypes,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PhotoUploadResponse:
        """
        Upload Whatsapp profile photo

        Args:
          file: Image file (JPEG recommended, max 10 MB)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not phone_number:
            raise ValueError(f"Expected a non-empty value for `phone_number` but received {phone_number!r}")
        body = deepcopy_minimal({"file": file})
        files = extract_files(cast(Mapping[str, object], body), paths=[["file"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return await self._post(
            f"/v2/whatsapp/phone_numbers/{phone_number}/profile/photo",
            body=await async_maybe_transform(body, photo_upload_params.PhotoUploadParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PhotoUploadResponse,
        )


class PhotoResourceWithRawResponse:
    def __init__(self, photo: PhotoResource) -> None:
        self._photo = photo

        self.delete = to_raw_response_wrapper(
            photo.delete,
        )
        self.upload = to_raw_response_wrapper(
            photo.upload,
        )


class AsyncPhotoResourceWithRawResponse:
    def __init__(self, photo: AsyncPhotoResource) -> None:
        self._photo = photo

        self.delete = async_to_raw_response_wrapper(
            photo.delete,
        )
        self.upload = async_to_raw_response_wrapper(
            photo.upload,
        )


class PhotoResourceWithStreamingResponse:
    def __init__(self, photo: PhotoResource) -> None:
        self._photo = photo

        self.delete = to_streamed_response_wrapper(
            photo.delete,
        )
        self.upload = to_streamed_response_wrapper(
            photo.upload,
        )


class AsyncPhotoResourceWithStreamingResponse:
    def __init__(self, photo: AsyncPhotoResource) -> None:
        self._photo = photo

        self.delete = async_to_streamed_response_wrapper(
            photo.delete,
        )
        self.upload = async_to_streamed_response_wrapper(
            photo.upload,
        )
